#!/bin/bash
# This script is part of the DAZL Web application
#
# ----------------------------------------------------------------------------
# For details on this code see the books "z/VM and Linux on IBM System z:
# The Virtualization Cookbook for <x>" on the Web at: http://linuxvm.org/present
# where <x> is "SLES 10" or "RHEL 5"
#
# This code is shipped under the Common Public License. See the file
# Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
# ----------------------------------------------------------------------------
#
#+--------------------------------------------------------------------------+
function cpCmd()
# echo a CP command and invoke it via the vmcp module/command
#   Arg1-n: the command to issue
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
# parse output to get return code: awk -F# splits line at '#' with rc at end
  output=`vmcp $@ 2>&1`
  ret_val=0
  ret_val=`echo $output | grep "Error: non-zero CP response" | awk -F# '{print $2}'`
  return $ret_val
 }

#+--------------------------------------------------------------------------+
function setup()
# Check for the existence of /backup/vm/USER.DISKMAP file, see if it has to
# be updated 
#+--------------------------------------------------------------------------+
 {
  DISKMAP="/backup/vm/USER.DISKMAP"
  # check if user wants to first refresh USER DISKMAP from MAINT 191 disk
  if [ $CONTENT_LENGTH != 0 ]; then # the one checkbox was set
    read args
    if [ $args = "refreshUserDiskmap=on" ]; then
      getUserDiskmap
    fi
  fi
  if [ ! -f $DISKMAP ]; then # just put up a file not found page
    echo "Content-type: text/html"
    echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
    echo '<html xmlns="http://www.w3.org/1999/xhtml">'
    echo ""
    echo "<head>"
    echo '<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />'
    echo "<title>DAZL DASD report</title>"
    echo "</head>"
    echo "<h4>The file $DISKMAP is not found</h4>"
    echo "args = $args"
    exit
  fi 
  NUM_LINES=`wc -l $DISKMAP | awk '{ print $1 }'`
 }

#+--------------------------------------------------------------------------+
function getUserDiskmap()
# echo a CP command and invoke it via the vmcp module/command
#   Arg1-n: the command to issue
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
  # if /backup/vm/ doesn't exist, then create it
  if [ ! -d /backup/vm ]; then # create it
    mkdir -p /backup/vm
    rc = $?
    if [ $rc != 0 ]; then # punt
      echo "/backup/vm is not a directory and cannot create it"
      echo "mkdir returned $rc"
      exit 1
    fi
  fi
  cd /backup/vm

  # link MAINT 191 to 1191 - if 1191 exists, assume it has USER DISKMAP file
  cpCmd QUERY VIRTUAL 1191
  rc=$?
  if [ $rc = 40 ]; then # 1191 is free 
    cpCmd LINK MAINT 191 1191 RR
    rc=$?
    if [ $rc != 0 ]; then # can't link MAINT 191 - punt
      echo "Can't link MAINT 191 read-only?"
      echo "LINK returned $rc"
      exit 2
    fi
    # else - fall through assuming 1191 is MAINT 191
  fi
  # activate 1191 then sleep for CP to catch up
  chccwdev -e 1191 > /dev/null 2>&1
  sleep 1

  # cat the USER DISKMAP file from VM to Linux
  cmsdevice=$(cat /proc/dasd/devices | grep 1191 | awk '{print $7}')
  cmsfscat -a -d /dev/$cmsdevice USER.DISKMAP > USER.DISKMAP

  # clean up
  chccwdev -d 1191 > /dev/null 2>&1
  cpCmd DET 1191
 } 

#+--------------------------------------------------------------------------+
function createHeader ()
# create the HTML header
#+--------------------------------------------------------------------------+
 {
  echo "Content-type: text/html"
  echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'
  echo '<html xmlns="http://www.w3.org/1999/xhtml">'
  echo ""
  echo "<head>"
  echo '<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />'
  echo "<title>Data About z/VM DASD</title>"
  echo "</head>"
 } # createHeader()

#+--------------------------------------------------------------------------+
function getFirstThreeMinidisks ()
# Get the first three minidisks in the volser 
#+--------------------------------------------------------------------------+
 {
  volSer=$1
  startLine=`grep -n "^$volSer" $DISKMAP | awk -F: '{ print $1 }'`
  if [ ! $startLine ]; then # volume not found
    echo "Volume $volSer not found"
  else
    let tailLines=$NUM_LINES-$startLine
    dashLine=`tail -$tailLines $DISKMAP | grep -n "^-" | head -1 | awk -F: '{ print $1 }'`
    if [ ! $dashLine ]; then # reached end of file
      let numDisks=$tailLines+1
    else 
      let numDisks=$dashLine-1
    fi

    # get the last used cylinder to be used in the "free/cyls" column
    if [ $numDisks = 1 ]; then
      endCylLine=$startLine
      lastCyl=`head -$startLine $DISKMAP | tail -1 | awk '{ print $6 }'`
    else
      let endCylLine=$startLine+numDisks-1
      lastCyl=`head -$endCylLine $DISKMAP | tail -1 | awk '{ print $5 }'`
    fi

    # if there are more than three minidisks and a link to the word "more"
    if (( $numDisks > 3 )); then
      let endLine=$startLine+3
      more=" | <a style=color:green href=/cgi-bin/getVolMap.sh?$volSer>More</a>"
    else
      let endLine=$startLine+$numDisks
      more=" "
    fi
    let i=$startLine
    while (( $i < $endLine )); do # gotta be a better way to print one line
      if [ $i = $startLine ]; then # this is the first line, skip volume name
        output=`head -$i $DISKMAP | tail -1 | awk '{ print $2,$3 }'`
      else # userID and vaddr are tokens 1 and 2
        output="$output | `head -$i $DISKMAP | tail -1 | awk '{ print $1,$2 }'`"
      fi
      let i=$i+1 
    done
    echo "$output $more"
  fi
 } # function getFirstThreeMinidisks

#+--------------------------------------------------------------------------+
function processStatus 
#
#+--------------------------------------------------------------------------+
 {
  echo "$status"
  let tailLines=$NUM_LINES-$startLine+2
  overlap=`tail -$tailLines $DISKMAP | head -$dashLine | grep "\*OVERLAP\*"`
  if [ ! $overlap ]; then
    echo " "
  else
    echo "<a style=color:red href=/cgi-bin/getVolMap.sh?$volSer>OVERLAP</a>"
  fi
 }

#+--------------------------------------------------------------------------+
function calculateSpace
# calculate the number of cylinders used and total. Also, list GAPs 
#+--------------------------------------------------------------------------+
 {
  numCyls=`sudo vmcp q da details $1 | head -1 | awk '{ print $13 }'`
  noZeros=${lastCyl#0*} # chop leading zeros off
  let usedCyls=$noZeros+1
  echo "$usedCyls/$numCyls"
  gap=`tail -$tailLines $DISKMAP | head -$dashLine | grep "GAP"`
  if [ ! $gap ]; then
    echo " "
  else
    echo "<a style=color:orange href=/cgi-bin/getVolMap.sh?$volSer>GAPs</a>"
  fi
 } # function calculateSpace

#+--------------------------------------------------------------------------+
function createRow ()
# create one row in the table
# arg 1: the volume serial number, aka label
#+--------------------------------------------------------------------------+
 {
  volSer=$1
  firstLine=`sudo vmcp query dasd $volSer | head -1`
  devAddr=`echo $firstLine | awk '{ print $2 }'`
  status=`echo $firstLine | awk '{ print $3, $4 }'`
  echo "<tr>"

  # Volume Label column
  echo "<td>"
  echo "<a style=color:green href=/cgi-bin/getVolMap.sh?$volSer>$volSer</a>"
  echo "</td>"

  # Device Address column
  echo "<td>"
  echo "<a style=color:green href=/cgi-bin/getDasdInfo.sh?$devAddr>$devAddr</a>"
  echo "</td>"

  # Minidisks column
  echo "<td>"
  echo "<tt>"
  getFirstThreeMinidisks $volSer
  echo "</tt>"
  echo "</td>"

  # Status column
  echo "<td>"
  echo "<tt>"
  processStatus $volSer
  echo "</tt>"
  echo "</td>"
 
  # Space column
  echo "<td>"
  echo "<tt>"
  calculateSpace $devAddr
  echo "</tt>"
  echo "</td>"

  # end of row
  echo "</tr>"
 }

#+--------------------------------------------------------------------------+
function createBody ()
# create the body of the HTML page
#+--------------------------------------------------------------------------+
 {
  echo "<body>"
  echo "<table border=2>"
  echo "<tr>"
  echo "<th>Volume Label</th>"
  echo "<th>Device Address</th>"
  echo "<th>Minidisks (first three)</th>"
  echo "<th>Status</th>"
  echo "<th>Cylinders used/total</th>"
  echo "</tr>"

  # loop through each volume in USER DISKMAP
  volumes=`egrep -v '^#|^$|^-|^VOLUME|^ |^\\\$' $DISKMAP | awk '{ print $1 }' | sort`
  for i in $volumes
  do
    createRow $i 
  done
 }
  
#+--------------------------------------------------------------------------+
function createFooter ()
# end the table and body
#+--------------------------------------------------------------------------+
 {
  echo "</table>"
  echo "</body>"
 }
  
# main()
setup
createHeader
createBody
createFooter
