#!/bin/sh
# This script is part of the DAZL Web application
#
# ----------------------------------------------------------------------------
# For details on this code see the books "z/VM and Linux on IBM System z:
# The Virtualization Cookbook for <x>" on the Web at: http://linuxvm.org/present
# where <x> is "SLES 10" or "RHEL 5"
#
# This code is shipped under the Common Public License. See the file
# Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
# ----------------------------------------------------------------------------
#
#---------------------------------------------------------------------------
function startPage()
# start the HTML page
# send the df command to the target clone - if there is an error then exit
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "Content-type: text/html"
  echo ""
  echo "<head>"
  echo "<h2>$linux information</h2>"
  df=`sudo ssh root@$linux df -h`
  rc=$?
  if [ $rc != 0 ]; then # report error
    echo "<title>Error running df on $linux</title>"
    echo "</head>"
    echo "<body>"
    echo "<h2>Error running df on $linux</h2>"
    echo "<pre>"
    echo "Return code from from \"sudo ssh root@$linux df -h\" = $rc"
    echo "It's possible key-based authentication to this server is not set up"
    echo "</pre>"
    echo "</body>"
    exit 1
  else # success
    echo "<title>$linux information</title>" 
    echo "</head>"
    echo "<body>"
  fi
 }

#---------------------------------------------------------------------------
function endPage()
#---------------------------------------------------------------------------
 {
  echo "</body>"
 }

#---------------------------------------------------------------------------
function startTable()
#---------------------------------------------------------------------------
 {
  echo "<table border=2>"
 }

#---------------------------------------------------------------------------
function endTable()
#---------------------------------------------------------------------------
 {
  echo "</table>"
 }

#---------------------------------------------------------------------------
function startRow()
#---------------------------------------------------------------------------
 {
  echo "<tr>"
 }

#---------------------------------------------------------------------------
function endRow()
#---------------------------------------------------------------------------
 {
  echo "</tr>"
 }

#---------------------------------------------------------------------------
function dfCell()
# put output of "df -h" in a table cell
#---------------------------------------------------------------------------
 {
  echo "<td>" 
  echo "<h4>Display file system disk usage: <tt>df -h</tt></h4>"
  echo "<pre>"
  echo "$df"
  echo "</pre>"
  echo "</td>" 
 }
  
#---------------------------------------------------------------------------
function mountCell()
# put output of "mount" in a table cell
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>" 
  echo "<h4>Display mounted file systems: <tt>mount</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux mount 
  echo "</pre>"
  echo "</td>" 
 }
  
#---------------------------------------------------------------------------
function runlevelCell()
# put output of "runlevel" in a table cell
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>" 
  echo "<h4>Display previous and current run level: <tt>runlevel</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux runlevel 
  echo "</pre>"
  echo "</td>" 
 }
  
#---------------------------------------------------------------------------
function unameCell()
# put output of "uname" information in a table cell
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  kernel=`uname -s`
  node=`uname -n`
  release=`uname -r`
  version=`uname -v`
  machine=`uname -m`
  processor=`uname -p`
  os=`uname -o`

  echo "<td>"
  echo "<h4>Display system information: <tt>uname</tt></h4>"
  echo "<pre>"
  echo "Kernel name:  $kernel"
  echo "Node name:  $node"
  echo "Kernel release:  $release"
  echo "Kernel version:  $version"
  echo "Machine hardware:  $machine"
  echo "Processor:  $processor"
  echo "Operating system:  $os"
  echo "</pre>"
  echo "</td>" 
 }
  
#---------------------------------------------------------------------------
function wCell()
# put output of "w" in a table cell
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>"
  echo "<h4>Show who is logged on: <tt>w</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux w
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function freeCell()
# an addictive card game
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>"
  echo "<h4>Display MBs of free and used memory: <tt>free -m</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux free -m
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function lsdasdCell()
# show the output of the lsdasd -s command in a table cell 
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>"
  echo "<h4>List Direct Access Storage Devices (DASD): <tt>lsdasd</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux lsdasd 
  echo "</pre>"
  echo "<h4>List all devices: <tt>ls /sys/bus/ccw/devices</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux ls -x /sys/bus/ccw/devices
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function pstreeCell()
# put output of pstree in a table cell 
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>"
  echo "<h4>Display a tree of processes: <tt>pstree</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux pstree
  echo "</pre>"
  echo "</td>"
 }

# main () 
startPage $1
startTable

startRow
dfCell
mountCell $1
endRow

startRow
runlevelCell $1
wCell $1
endRow

startRow
unameCell $1
freeCell $1
endRow

startRow
pstreeCell $1
lsdasdCell $1
endRow

# finish up
endTable
endPage
