#!/bin/sh
# This script is part of the DAZL Web application
#
# ----------------------------------------------------------------------------
# For details on this code see the books "z/VM and Linux on IBM System z:
# The Virtualization Cookbook for <x>" on the Web at: http://linuxvm.org/present
# where <x> is "SLES 10" or "RHEL 5"
#
# This code is shipped under the Common Public License. See the file
# Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
# ----------------------------------------------------------------------------
#
#---------------------------------------------------------------------------
function startPage()
# start the HTML page
# send the df command to the target clone - if there is an error then exit
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "Content-type: text/html"
  echo ""
  echo "<head>"
  echo "<h2>$linux network info</h2>"
  ifconfig=`sudo ssh root@$linux ifconfig eth0`
  rc=$?
  if [ $rc != 0 ]; then # report error
    echo "<title>Error running ifconfig on $linux</title>"
    echo "</head>"
    echo "<body>"
    echo "<h2>Error running ifconfig on $linux</h2>"
    echo "<pre>"
    echo "Return code from from \"sudo ssh root@$linux ifconfig\" = $rc"
    echo "It's possible key-based authentication to this server is not set up"
    echo "</pre>"
    echo "</body>"
    exit 1
  else # success
    echo "<title>$linux network info</title>" 
    echo "</head>"
    echo "<body>"
  fi
 }

#---------------------------------------------------------------------------
function endPage()
#---------------------------------------------------------------------------
 {
  echo "</body>"
 }

#---------------------------------------------------------------------------
function startTable()
#---------------------------------------------------------------------------
 {
  echo "<table border=2>"
 }

#---------------------------------------------------------------------------
function endTable()
#---------------------------------------------------------------------------
 {
  echo "</table>"
 }

#---------------------------------------------------------------------------
function startRow()
#---------------------------------------------------------------------------
 {
  echo "<tr>"
 }

#---------------------------------------------------------------------------
function endRow()
#---------------------------------------------------------------------------
 {
  echo "</tr>"
 }

#---------------------------------------------------------------------------
function ifconfigCell()
# put output of "ifconfig eth0" in a table cell
#---------------------------------------------------------------------------
 {
  echo "<td>" 
  echo "<h4>Display eth0 interface information: <tt>ifconfig eth0</tt></h4>"
  echo "<pre>"
  echo "$ifconfig"
  echo "</pre>"
  echo "</td>" 
 }
  
#---------------------------------------------------------------------------
function iplinkCell()
# put output of "ip link show" in a table cell
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>" 
  echo "<h4>Display IP link informaton: <tt>ip link show</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux ip link show
  echo "</pre>"
  echo "</td>" 
 }
  
#---------------------------------------------------------------------------
function netstatCell()
# put output of "netstat -tan" in a table cell
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>" 
  echo "<h4>Display TCP network information: <tt>netstat -tan</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux netstat -tan 
  echo "</pre>"
  echo "</td>" 
 }
  
#---------------------------------------------------------------------------
function iptablesCell()
# put output of "iptables -L" in a table cell
# arg 1: the target Linux system
#---------------------------------------------------------------------------
 {
  linux=$1
  echo "<td>"
  echo "<h4>List IP table rules: <tt>iptables -l</tt></h4>"
  echo "<pre>"
  sudo ssh root@$linux iptables -L 
  echo "</pre>"
  echo "</td>"
 }

# main () 
startPage $1
startTable

startRow
ifconfigCell
iplinkCell $1
endRow

startRow
netstatCell $1
iptablesCell $1
endRow

# finish up
endTable
endPage
