#!/bin/bash
# This script is part of the DAZL Web application
# ----------------------------------------------------------------------------
# For details on this code see the books "z/VM and Linux on IBM System z:
# The Virtualization Cookbook for <x>" on the Web at: http://linuxvm.org/present
# where <x> is "SLES 10" or "RHEL 5"
#
# This code is shipped under the Common Public License. See the file
# Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
# ----------------------------------------------------------------------------
#
# getUserDirect.sh - copy USER DISKMAP file from MAINT 191 to /backup/vm
#+--------------------------------------------------------------------------+
function cp_cmd()
# echo a CP command and invoke it via the vmcp module/command
#   Arg1-n: the command to issue
#   Return: the command's return code
#+--------------------------------------------------------------------------+
 {
  echo "Invoking CP command: $@"
# parse output to get return code: awk -F# splits line at '#' with rc at end
  output=`vmcp $@ 2>&1`
  echo "$output"
  ret_val=0
  ret_val=`echo $output | grep "Error: non-zero CP response" | awk -F# '{print $2}'`
  return $ret_val
 }

# main()
# if /backup/vm/ doesn't exist, then create it
if [ ! -d /backup/vm ]; then # create it
  mkdir -p /backup/vm
  rc = $?
  if [ $rc != 0 ]; then # punt
    echo "/backup/vm is not a directory and cannot create it"
    echo "mkdir returned $rc"
    exit
  fi
fi
cd /backup/vm

# link MAINT 191 at vdev 1191 - if 1191 exists, assume it has USER DISKMAP file
cp_cmd QUERY VIRTUAL 1191
rc=$?
if [ $rc = 40 ]; then # 1191 is free 
  cp_cmd LINK MAINT 191 1191 RR
  rc=$?
  if [ $rc != 0 ]; then # can't link MAINT 191 - punt
    echo "Can't link MAINT 191 read-only?"
    echo "LINK returned $rc"
    exit
  fi
  # else - fall through assuming 1191 is MAINT 191
fi
# activate 1191 then sleep for CP to catch up
chccwdev -e 1191
sleep 1

# read the USER DISKMAP file
cmsdevice=$(cat /proc/dasd/devices | grep 1191 | awk '{print $7}')
cmsfscat -a -d /dev/$cmsdevice USER.DISKMAP > USER.DISKMAP

# clean up
chccwdev -d 1191
cp_cmd DET 1191
