#!/bin/sh
# This script is part of the DAZL Web application
#
# ----------------------------------------------------------------------------
# For details on this code see the books "z/VM and Linux on IBM System z:
# The Virtualization Cookbook for <x>" on the Web at: http://linuxvm.org/present
# where <x> is "SLES 10" or "RHEL 5"
#
# This code is shipped under the Common Public License. See the file
# Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
# ----------------------------------------------------------------------------
#
#---------------------------------------------------------------------------
function processArgs()
# process the arguments passed into this CGI-BIN script
# arg 1: z/VM user ID 
#---------------------------------------------------------------------------
 {
  args=`echo $QUERY_STRING | sed 's/&/ /g'`
  userID=`echo $args | awk '{ print $1 }'`
 }

#---------------------------------------------------------------------------
function createHeader()
# Create Web page header 
#---------------------------------------------------------------------------
 {
  # create the page header
  echo "Content-type: text/html"
  echo ""
  echo "<head>"
  echo "<title>z/VM user ID $userID</title>"
  echo "</head>"
  echo "<body>"
 }

#---------------------------------------------------------------------------
function endPage()
#---------------------------------------------------------------------------
 {
  echo "</body>"
 }

#---------------------------------------------------------------------------
function startTable()
#---------------------------------------------------------------------------
 {
  echo "<table border=2>"
 }

#---------------------------------------------------------------------------
function endTable()
#---------------------------------------------------------------------------
 {
  echo "</table>"
 }

#---------------------------------------------------------------------------
function startRow()
#---------------------------------------------------------------------------
 {
  echo "<tr>"
 }

#---------------------------------------------------------------------------
function endRow()
#---------------------------------------------------------------------------
 {
  echo "</tr>"
 }

#---------------------------------------------------------------------------
function dirEntryCell()
# create the directory entry section
#---------------------------------------------------------------------------
 {
  echo "<td>"
  echo "<h4>$userID entry in USER DIRECT:</h4>"
  echo "<pre>"
  grep -q "^USER $userID" $BACKUPVM/USER.DIRECT
  if [ $? != 0 ]; then # not found
    echo "USER $userID not found at start of line in $BACKUP/USER.DIRECT"
  else # extract the directory entry - this is ugly - gotta be an easier way (???)
    startLine=`grep -n "^USER $userID" $BACKUPVM/USER.DIRECT | awk -F: '{ print $1 }'`
    numLines=`wc -l $BACKUPVM/USER.DIRECT | awk '{ print $1 }'`
    let tailLines=$numLines-$startLine
    linesInEntry=`tail -$tailLines $BACKUPVM/USER.DIRECT | grep -n "^USER" | head -1 | awk -F: '{ print $1 }'`
    let tailLines=$tailLines+2 # back up to one line BEFORE the USER statement
    tail -$tailLines $BACKUPVM/USER.DIRECT | head -$linesInEntry
  fi
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function indUserCell()
# create the directory entry section
#---------------------------------------------------------------------------
 {
  echo "<td>"
  echo "<h4>Display resources used by $userID: <tt>IND USER $userID</tt></h4>"
  echo "<pre>"
  sudo vmcp IND USER $userID 
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function parmFileCell()
# create a cell of the user ID's PARMFILE file if it exists
#---------------------------------------------------------------------------
 {
  parmFile=$BACKUPVM/$userID.PARMFILE
  echo "<td>"
  echo "<h4>SLES 9 parameter file: $userID PARMFILE:</h4>"
  echo "<pre>"
  if [ -f $parmFile ]; then # just type it
    cat $parmFile
  else
    echo "$parmFile not found"
  fi
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function parmS10Cell()
# create a cell of the user ID's PARM-S10 file if it exists
#---------------------------------------------------------------------------
 {
  parmS10=$BACKUPVM/$userID.PARM-S10
  echo "<td>"
  echo "<h4>SLES 10 parameter file: $userID PARM-S10:</h4>"
  echo "<pre>"
  if [ -f $parmS10 ]; then # just type it
    cat $parmS10
  else
    echo "$parmS10 not found"
  fi
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function parmRH5Cell()
# create a cell of the user ID's PARM-RH5 file if it exists
#---------------------------------------------------------------------------
 {
  parmRH5=$BACKUPVM/$userID.PARM-RH5
  echo "<td>"
  echo "<h4>RHEL 5 parameter file: $userID PARM-RH5:</h4>"
  echo "<pre>"
  if [ -f $parmRH5 ]; then # just type it
    cat $parmRH5
  else
    echo "$parmRH5 not found"
  fi
  echo "</pre>"
  echo "</td>"
 }

#---------------------------------------------------------------------------
function confRH5Cell()
# create a cell of the user ID's CONF-RH5 file if it exists
#---------------------------------------------------------------------------
 {
  confRH5=$BACKUPVM/$userID.CONF-RH5
  echo "<td>"
  echo "<h4>RHEL 5 configuration file: $userID CONF-RH5:</h4>"
  echo "<pre>"
  if [ -f $confRH5 ]; then # just type it
    cat $confRH5
  else
    echo "$confRH5 not found"
  fi
  echo "</pre>"
  echo "</td>"
 }

# main()
BACKUPVM="/backup/vm" # directory where z/VM data is backed up
processArgs
createHeader
startTable

startRow
dirEntryCell
indUserCell
endRow

startRow
parmFileCell
parmS10Cell
endRow

startRow
parmRH5Cell
confRH5Cell
endRow

endTable
endPage

