// This code is part of the DAZL Web Application
// 
// For details on this code see the books "z/VM and Linux on IBM System z:
// The Virtualization Cookbook for <x> on the Web at: http://linuxvm.org/present
// where <x> is "-S10" for SLES 10 or "-RH5" for RHEL 5 
//
// This code is shipped under the Common Public License. See the file
// Common-Public-License.txt or http://www.opensource.org/licenses/cpl1.0.txt
// ----------------------------------------------------------------------------
//
//--------------------------------------------------------------------------
// EditLog class - singleton to modify one text field at a time in place
//--------------------------------------------------------------------------
var EditLog = new function()
 {
  this.textarea = null;      // pointer to input text field with desc/log
  this.logFileName = null;   // pointer to hidden field with name of log file 
  this.cancelButton = null;  // pointer to Cancel button (when it exists) 
  this.cancelDiv = null;     // pointer to Cancel div (when it exists) 
  this.saveButton = null;    // pointer to Save button (when it exists) 
  this.saveDiv = null;       // pointer to Save div (when it exists) 
  this.editing = null;       // 'ro'-view desc, or 'rw'-edit log
  this.message = null;       // message to put in alert 
  
  //--------------------------------------------------------------------------
  // Constructor
  this.init = function()
  //--------------------------------------------------------------------------
   {
    var self = EditLog;
    self.editing = 'ro';
    self.message = 'Not set';
   };

  //--------------------------------------------------------------------------
  // delete the two buttons and set 
  this.restoreDesc = function()
  //--------------------------------------------------------------------------
   {
    // delete the cancel and save buttons and set rows back to 1
    var self = EditLog;
    self.saveDiv.removeChild(self.saveButton);
    self.logDiv.removeChild(self.saveDiv);
    self.cancelDiv.removeChild(self.cancelButton);
    self.logDiv.removeChild(self.cancelDiv);
    // pass the file name to getDesc.sh and the first line is returned
    fleegix.xhr.doPost(self.textareaHandler, '/cgi-bin/getDesc.sh', self.logFileName.value);
    self.textarea.rows = 1;
   };

  //--------------------------------------------------------------------------
  // Save changes to log file - the format mirrors the cgi-bin saveLog.sh
  // First line is file name, middle lines are contents, last line is "EoF"
  this.doSave = function()
  //--------------------------------------------------------------------------
   {
    var self = EditLog;
    var string = self.logFileName.value + '\n' + self.textarea.value + '\nEoF';
    fleegix.xhr.doPost(self.saveHandler, '/cgi-bin/saveLog.sh', string);
    self.editing = 'ro'; 
   };

  //--------------------------------------------------------------------------
  // discard the changed and restore the one line text input field
  this.doCancel = function()
  //--------------------------------------------------------------------------
   {
    // restore the original description
    var self = EditLog;
    self.editing = 'ro'; 
    self.restoreDesc();
   };
  
  //--------------------------------------------------------------------------
  // get the log file - get the contents of the log file from the server
  // add Save and Cancel buttons 
  this.getLog = function(theObject) 
  //--------------------------------------------------------------------------
   {
    var self = EditLog;
    if (self.editing == 'rw') // then return - can only edit one at a time
     {
      return;
     }
    self.textarea = theObject;
    self.editing = 'rw';
    self.logFileName = document.getElementById(self.textarea.id + 'file');
    fleegix.xhr.doPost(self.textareaHandler, '/cgi-bin/getLog.sh', self.logFileName.value);
    self.textarea.rows = 10;           // enlarge the text area 
  
    // create a Save button and div on the left
    self.saveDiv = document.createElement('div');
    self.saveDiv.className = 'leftButton';
    self.saveButton = document.createElement('input');
    self.saveButton.className = 'inputButton';
    self.saveButton.type = 'button';
    self.saveButton.onclick = self.doSave; 
    self.saveButton.id = 'saveButtonID'; 
    self.saveButton.value = 'Save';
    self.saveDiv.appendChild(self.saveButton);

    // create a Cancel button and div on the right
    self.cancelDiv = document.createElement('div');
    self.cancelDiv.className = 'rightButton';
    self.cancelButton = document.createElement('input');
    self.cancelButton.className = 'inputButton';
    self.cancelButton.type = 'button';
    self.cancelButton.onclick = self.doCancel; 
    self.cancelButton.id = 'cancelButtonID' 
    self.cancelButton.value = 'Cancel';
    self.cancelDiv.appendChild(self.cancelButton);

    // find the log div and append the divs to it 
    self.logDiv = document.getElementById(theObject.id + 'div');
    self.logDiv.appendChild(self.saveDiv);
    self.logDiv.appendChild(self.cancelDiv);
   };

  //--------------------------------------------------------------------------
  // alert response handler - pop up the return string in an alert
  this.saveHandler = function(string)
  //--------------------------------------------------------------------------
   {
    var self = EditLog;
    alert(string);
    self.editing = 'ro';
    self.restoreDesc();  // restore the description line
   };

  //--------------------------------------------------------------------------
  // textarea response handler - put the return string in the current textarea
  this.textareaHandler = function(string) 
  //--------------------------------------------------------------------------
   {
    var self = EditLog;
    self.textarea.value = string;
   };
 };

window.onload = EditLog.init;
